#! /usr/bin/env python

# $Id: $

__author__ = 'Greig A Cowan'
__date__ = 'January 2008'
__version__ = 0.1

import sys, os
import re
import ldap
import pg
from optparse import OptionParser

def main():
    parser = OptionParser(
        usage = 'usage: %prog GOCDB-site-name')
    
    (options, args) = parser.parse_args()
    
    site = ''
    try:
        site = args[0]
    except ValueError:
        parser.print_help()
        parser.exit()                        
            
    server = 'lcg-bdii.cern.ch:2170'
    who = ''
    cred = ''

    try:
        l = ldap.open( server)
        l.simple_bind_s( who, cred)
        print 'Successfully bound to server.'
        print 'Searching..\n'
    except ldap.LDAPError, error_message:
        print 'Could not connect. %s ' % error_message

    sites = get_sites( l, site)
    for site in sites:
        name = site[0][1]['GlueSiteUniqueID'][0]
        ses = get_SEs( l, name)
        for se in ses:
            host = se[0][1]['GlueSEUniqueID'][0]
            print host
            sas = get_SAs( l, host)
            tokens = get_VOInfo( l, host)
            for sa in sas:
                localID = sa[0][1]['GlueSALocalID'][0]
                for token in tokens:
                    for i in token[0][1]['GlueChunkKey']:
                        if i == 'GlueSALocalID='+localID:
                            insert = [ name, host,
                                       token[0][1]['GlueVOInfoName'][0].split(':')[0],
                                       token[0][1]['GlueVOInfoTag'][0],
                                       sa[0][1]['GlueSAStateAvailableSpace'][0],
                                       sa[0][1]['GlueSAStateUsedSpace'][0],
                                       sa[0][1]['GlueSATotalOnlineSize'][0],
                                       sa[0][1]['GlueSAUsedOnlineSize'][0],
                                       sa[0][1]['GlueSAFreeOnlineSize'][0],
                                       sa[0][1]['GlueSAReservedOnlineSize'][0],
                                       sa[0][1]['GlueSATotalNearlineSize'][0],
                                       sa[0][1]['GlueSAUsedNearlineSize'][0],
                                       sa[0][1]['GlueSAFreeNearlineSize'][0],
                                       sa[0][1]['GlueSAReservedNearlineSize'][0]]
                    
                            put( insert)
                                
def get_sites( l, site):
    base = 'o=grid'
    scope = ldap.SCOPE_SUBTREE
    filter = 'GlueSiteUniqueID=%s' % site
    retrieve_attributes = ['GlueSiteUniqueID']
    return search( l, base, scope, filter, retrieve_attributes)

def get_SEs( l, site):
    base = 'o=grid'
    scope = ldap.SCOPE_SUBTREE
    # It seems that some sites (all dCache?) are not publishing GlueForeignKey in this way
    filter = '(&(GlueSEUniqueID=*)(GlueForeignKey=GlueSiteUniqueID=%s))' % site
    retrieve_attributes = ['GlueSEUniqueID',
                           'GlueSEImplementationName',
                           'GlueSEImplementationVersion',
                           'GlueSEArchitecture',
                           'GlueSESizeFree',
                           'GlueSESizeTotal'
                           'GlueSEUsedOnlineSize',
                           'GlueSEUsedNearlineSize',
                           'GlueSETotalOnlineSize',
                           'GlueSchemaVersionMajor',
                           'GlueSchemaVersionMinor']

    return search( l, base, scope, filter, retrieve_attributes)

def get_SAs( l, se):
    base = 'o=grid'
    scope = ldap.SCOPE_SUBTREE
    filter = '(&(GlueSALocalID=*)(GlueChunkKey=GlueSEUniqueID=%s))' % se
    retrieve_attributes = ['GlueSALocalID',
                           'GlueSAPath',
                           'GlueSAStateAvailableSpace',
                           'GlueSAStateUsedSpace',
                           'GlueSAAccessControlBaseRule',
                           'GlueSARetentionPolicy',
                           'GlueSAAccessLatency',
                           'GlueSATotalOnlineSize',
                           'GlueSAUsedOnlineSize',
                           'GlueSAFreeOnlineSize',
                           'GlueSAReservedOnlineSize',
                           'GlueSATotalNearlineSize',
                           'GlueSAUsedNearlineSize',
                           'GlueSAFreeNearlineSize',
                           'GlueSAReservedNearlineSize',
                           'GlueChunkKey']

    return search( l, base, scope, filter, retrieve_attributes)

def get_VOInfo( l, se):
    base = 'o=grid'
    scope = ldap.SCOPE_SUBTREE
    filter = '(&(GlueVOInfoLocalID=*)\
    (GlueChunkKey=GlueSEUniqueID=%s)\
    (!(GlueVOInfoTag=UNAVAILABLE)))'\
    % ( se)

    retrieve_attributes = ['GlueVOInfoName',
                           'GlueVOInfoTag',
                           'GlueChunkKey']
                    
    return search( l, base, scope, filter, retrieve_attributes)
               
def search( l, base, scope, filter, retrieve_attributes):
    result_set = []
    timeout = 0
    try:
        result_id = l.search(base, scope, filter, retrieve_attributes)
        while True:
            result_type, result_data = l.result(result_id, timeout)
            if (result_data == []):
                break
            else:
                if result_type == ldap.RES_SEARCH_ENTRY:
                    result_set.append(result_data)
    except ldap.LDAPError, error_message:
        print error_message
    return result_set

def put( token):
        
    db = pg.connect(dbname='srmversions', host='localhost',user='postgres')
    try:
        oid = db.query("""INSERT INTO tokenspace VALUES ('%s','%s','%s',\
        '%s','%s','%s','%s','%s','%s','%s','%s','%s','%s','%s')"""
                       % tuple( token))
    except pg.ProgrammingError, error_message:
        print 'Prog error:', error_message
    except pg.InternalError, error_message:
        print 'Internal error:', error_message
    except ValueError, error_message:
        print 'Value error:', error_message
    except TypeError, error_message:
        print 'Type error:', error_message
                
if __name__=='__main__':
    main()
