#! /usr/bin/env python

import sys, os
import subprocess
import re
import ldap
import threading
import Queue
import pexpect
import time
import pg

def main():
    server = 'lcg-bdii.cern.ch:2170'
    who = ''
    cred = ''
    endpoints = []
    try:
        l = ldap.open( server)
        l.simple_bind_s( who, cred)
        print 'Successfully bound to server.'
        print 'Searching..\n'
        endpoints = get_endpoints( l)
    except ldap.LDAPError, error_message:
        print 'Could not connect. %s ' % error_message
        
    endpointPool = Queue.Queue()

    for endpoint in endpoints:
        print endpoint
        endpointPool.put( endpoint)

    for i in range(5):
        t = SrmVersion(endpointPool)
        t.setDaemon(True)
        t.start()

    endpointPool.join() # only in py 2.5


def get_endpoints(l):
    regexp = re.compile('(httpg[\:\/\w\d\.\-]+)')
    base = 'mds-vo-name=local,o=grid'
    scope = ldap.SCOPE_SUBTREE
    filter = '(&(GlueServiceUniqueID=*)(objectClass=GlueService)\
    (GlueServiceType=SRM)(objectClass=glueService)\
    (GlueServiceVersion=2.2.0))'
    retrieve_attributes = ['GlueServiceEndpoint']
    count = 0
    result_set = []
    srm_endpoints = []
    timeout = 0
    try:
        result_id = l.search(base, scope, filter, retrieve_attributes)
        while True:
            result_type, result_data = l.result(result_id, timeout)
            if (result_data == []):
                break
            else:
                if result_type == ldap.RES_SEARCH_ENTRY:
                    result_set.append(result_data)
    except ldap.LDAPError, error_message:
        print error_message

    for result in result_set:
        host = regexp.match( result[0][1]['GlueServiceEndpoint'][0])
        srm_endpoints.append( host.group(1) )
    return uniq(srm_endpoints)

class SrmVersion( threading.Thread):
    def __init__( self, endpointPool):
        self.endpointPool = endpointPool
        self.db = pg.connect(dbname='srmversions', host='localhost',user='postgres')
        self.empty = False
        threading.Thread.__init__(self)
        
    def run( self):
        while not self.empty:
            try:
                endpoint = self.endpointPool.get_nowait()
                if endpoint != None:
                    self.getVersion( endpoint)
                    self.endpointPool.task_done()
            except Queue.Empty:
                print self.getName(), 'reports that there are no more endpoints.' 
                self.empty = True

    def getVersion( self, endpoint):
        val = re.compile('\s\s\s\s\[\d\] value=\"([\w\d\.\-\_]*)\"')
        srmv = re.compile('\s\s\s\s\[\d\] key=\"SRM version\"')
        bver = re.compile('\s\s\s\s\[\d\] key=\"backend_version\"')
        btyp = re.compile('\s\s\s\s\[\d\] key=\"backend_type\"')
        srmping = '/home/gcowan/opt/srmv2storm/bin/clientSRM \
        ping -e ' + endpoint
        version = ''
        type = ''
        srmversion = ''
        foundVersion = False
        foundType = False
        foundSrm = False
        try:
            child = pexpect.spawn(srmping)
            i = child.expect(['\"CGSI-gSOAP\: [\w\s\:\!]*\"',
                              pexpect.EOF,
                              pexpect.TIMEOUT],timeout=30)
            if i == 0:
                print endpoint, child.after
            elif i == 1:
                for line in child.before.split('\n'):
                    v = val.match(line)
                    ver = bver.match(line)
                    typ = btyp.match(line)
                    srm = srmv.match(line)
                    if v != None and foundVersion:
                        version = v.group(1)
                        foundVersion = False
                    if v != None and foundType:
                        type = v.group(1)
                        foundType = False
                    if v != None and foundSrm:
                        srmversion = v.group(1)
                        foundSrm = False
                    if ver != None:
                        foundVersion = True
                    if typ != None:
                        foundType = True
                    if srm != None:
                        foundSrm = True
                if srmversion == '':
                    self.putVersion( endpoint, version, type)
                else:
                    self.putVersion( endpoint, version+'-'+srmversion, type)
            else:
                print endpoint, 'Timeout'
        except OSError, error_message:
            print 'Exception' + error_message

        return version, type

    def putVersion(self, endpoint, version, type):
        try:
            oid = self.db.query("""INSERT INTO
            versions VALUES ('%s', '%s', '%s')"""
                                % (endpoint, version, type))
        except pg.ProgrammingError, error_message:
            print 'Prog error:', error_message
        except pg.InternalError, error_message:
            print 'Internal error:', error_message
        except ValueError, error_message:
            print 'Value error:', error_message
        except TypeError, error_message:
            print 'Type error:', error_message
        print endpoint + '\t' + version + '\t' + type

def uniq(seq):
    # Not order preserving
    keys = {}
    for e in seq:
        keys[e] = 1
    return keys.keys()
                        

if __name__=='__main__':
    main()

                #bt = re.compile('backend_type:([\w\d\.\-]*)')
                #bv = re.compile('backend_version:([\w\d\.\-]*)')
                #srmping = '/home/gcowan/opt/d-cache/srm/bin/srmping'
                #args = '-2 -debug ' + endpoint
                    #stdout, stderr = \
                    #        subprocess.Popen(\
                    #    [ srmping, args], stdout=subprocess.PIPE,\
                    #    stderr=subprocess.PIPE).communicate()
